#ifndef SAMPLINGSYMBOLE_H
#define SAMPLINGSYMBOLE_H

/*!
 * \file SamplingSymbole.h
 * \brief Reprsentation d'un symbole avec les diverses formes le composant
 *        et les mthodes d'chantillonnage.
 * \author Christophe Guinchard
 * \version 1.0
 */

#define PI 3.14159265

#include <string>
#include <vector>
#include <math.h>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <iterator>
#include "SamplingFormeCircle.h"
#include "SamplingFormeLine.h"
#include "SamplingFormeBezierCubique.h"
#include "SamplingFormeBezierQuadratique.h"
#include "SamplingFormeEllipse.h"

using namespace std;

/*! \class SamplingSymbole
 * \brief Classe reprsentant un symbole avec les diverses formes le
 *        composant et les mthodes d'chantillonnage.
 *
 *  Pour chaque symbole, on distinguera ses 5 formes lmentaires le
 *  reprsentant : les lignes, les cercles, les bziers cubiques, les
 *  bziers quadratiques et les ellispses. La classe contient une liste
 *  pour chacun de ces lements qui ont t parss du fichier SVG.
 *  Chacune de ces formes sont ensuite chantillonne en une liste de
 *  pooints.
 */
class SamplingSymbole
{

private:
	vector <SamplingFormeLine>				ligneListe;
	vector <SamplingFormeCircle>			cercleListe;
	vector <SamplingFormeBezierCubique>		bCubiqueListe;
	vector <SamplingFormeBezierQuadratique>	bQuadratiqueListe;
	vector <SamplingFormeEllipse>			ellipseListe;
	vector <pair <double, double>>			pointListe;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe SamplingSymbol
     */
	SamplingSymbole(void);

    /*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe SamplingSymbol
     */
	~SamplingSymbole(void);

public:

	/*!
     *  \brief Getter de point chantillonn
     *
     *  Methode qui permet de rcuprer un point chantillonn de la liste
	 *  pointListe.
	 *
     *  \param place : Emplacement du point  rcuprer dans la liste.
     *  \return le point de la liste. Ce point est une pair.
     */
	pair <double, double>			getPoint(unsigned int place){return pointListe.at(place);};

	/*!
     *  \brief Getter d'un lment ligne
     *
     *  Methode qui permet de rcuprer un lment ligne, une instance de la
	 *  classe SamplingFormeLine, dans la liste ligneListe.
	 *
     *  \param place : Emplacement de la ligne  rcuprer dans la liste.
     *  \return l'instance de la classe SamplingFormeLine.
     */
	SamplingFormeLine				getLigne(unsigned int place){return ligneListe.at(place);};

	/*!
     *  \brief Getter d'un lment cercle
     *
     *  Methode qui permet de rcuprer un lment cercle, une instance de la
	 *  classe SamplingFormeCircle, dans la liste cercleListe.
	 *
     *  \param place : Emplacement du cercle  rcuprer dans la liste.
     *  \return l'instance de la classe SamplingFormeCircle.
     */
	SamplingFormeCircle				getCercle(unsigned int place){return cercleListe.at(place);};

	/*!
     *  \brief Getter d'un lment bzier cubique
     *
     *  Methode qui permet de rcuprer un lment bzier cubique, une
	 *  instance de la classe SamplingFormeBezierCubique, dans la liste
	 *  bCubiqueListe.
	 *
     *  \param place : Emplacement du bzier cubique  rcuprer dans la liste.
     *  \return l'instance de la classe SamplingFormeBezierCubique.
     */
	SamplingFormeBezierCubique		getBezierCubique(unsigned int place){return bCubiqueListe.at(place);};

	/*!
     *  \brief Getter d'un lment bzier quadratique
     *
     *  Methode qui permet de rcuprer un lment bzier quadratique, une
	 *  instance de la classe SamplingFormeBezierQuadratique, dans la liste
	 *  bQuadratiqueListe.
	 *
     *  \param place : Emplacement du bzier quadatique  rcuprer dans la
	 *  liste.
     *  \return l'instance de la classe SamplingFormeBezierQuadratique.
     */
	SamplingFormeBezierQuadratique	getBezierQuadratique(unsigned int place){return bQuadratiqueListe.at(place);};

	/*!
     *  \brief Getter d'un lment llipse
     *
     *  Methode qui permet de rcuprer un lment llipse, une instance de la
	 *  classe SamplingFormeEllipse, dans la liste ellipseListe.
	 *
     *  \param place : Emplacement de l'llipse  rcuprer dans la liste.
     *  \return l'instance de la classe SamplingFormeEllipse
     */
	SamplingFormeEllipse			getEllipse(unsigned int place){return ellipseListe.at(place);};

	/*!
     *  \brief Getter du nombre de points
     *
     *  Methode qui permet de rcuprer le nombre de point chantillonns
	 *  de la liste pointListe.
	 *
     *  \return nombre de points.
     */
	unsigned int	getNombrePoint(void){return pointListe.size();};

	/*!
     *  \brief Getter du nombre de lignes
     *
     *  Methode qui permet de rcuprer le nombre de lignes de la liste
	 *  ligneListe.
	 *
     *  \return nombre de lignes.
     */
	unsigned int	getNombreLigne(void){return ligneListe.size();};

	/*!
     *  \brief Getter du nombre de cercles
     *
     *  Methode qui permet de rcuprer le nombre de cercles de la liste
	 *  cercleListe.
	 *
     *  \return nombre de cercles.
     */
	unsigned int	getNombreCercle(void){return cercleListe.size();};

	/*!
     *  \brief Getter du nombre de bziers cubiques
     *
     *  Methode qui permet de rcuprer le nombre de bziers cubiques de
	 *  la liste bCubiqueListe.
	 *
     *  \return nombre de bziers cubiques.
     */
	unsigned int	getNombreBezierCubique(void){return bCubiqueListe.size();};

	/*!
     *  \brief Getter du nombre de bziers quadratiques
     *
     *  Methode qui permet de rcuprer le nombre de bziers quadratiques
	 *  de la liste bQuadratiqueListe.
	 *
     *  \return nombre de bziers quadratique.
     */
	unsigned int	getNombreBezierQuadratique(void){return bQuadratiqueListe.size();};

	/*!
     *  \brief Getter du nombre d'llipses
     *
     *  Methode qui permet de rcuprer le nombre d'llipses de la liste
	 *  ellipseListe.
	 *
     *  \return nombre d'llipses.
     */
	unsigned int	getNombreEllipse(void){return ellipseListe.size();};

	/*!
     *  \brief Ajout de ligne
     *
     *  Methode qui permet d'ajouter une ligne  la liste ligneListe.
	 *
     *  \param x1 : Coordonne sur l'axe x du premier point de la ligne.
     *  \param y1 : Coordonne sur l'axe y du premier point de la ligne.
     *  \param x2 : Coordonne sur l'axe x du dernier point de la ligne.
     *  \param y2 : Coordonne sur l'axe y du dernier point de la ligne.
     */
	void	addLigne(double x1, double y1, double x2, double y2);

	/*!
     *  \brief Ajout de cercle
     *
     *  Methode qui permet d'ajouter un cercle  la liste cercleListe.
	 *
     *  \param cx : Coordonne sur l'axe x du centre du cercle.
     *  \param cy : Coordonne sur l'axe y du centre du cercle.
     *  \param rayon : Rayon du cercle.
     */
	void	addCercle(double cx, double cy, double rayon);

	/*!
     *  \brief Ajout de bzier cubique
     *
     *  Methode qui permet d'ajouter un bzier cubique  la liste
	 *  bCubiqueListe.
	 *
     *  \param x1 : Coordonne sur l'axe x du premier point du bzier.
     *  \param y1 : Coordonne sur l'axe y du premier point du bzier.
     *  \param cx1 : Coordonne sur l'axe x du premier point de controle.
     *  \param cy1 : Coordonne sur l'axe y du premier point de controle.
     *  \param cx2 : Coordonne sur l'axe x du second point de controle.
     *  \param cy2 : Coordonne sur l'axe y du second point de controle.
     *  \param x2 : Coordonne sur l'axe x du dernier point du bzier.
     *  \param y2 : Coordonne sur l'axe y du dernier point du bzier.
     */
	void	addBezierCubique(double x1, double y1, double cx1, double cy1, double cx2, double cy2, double x2, double y2);

	/*!
     *  \brief Ajout de bzier quadratique
     *
     *  Methode qui permet d'ajouter un bzier quadratique  la liste
	 *  bQuadratiqueListe.
	 *
     *  \param x1 : Coordonne sur l'axe x du premier point du bzier.
     *  \param y1 : Coordonne sur l'axe y du premier point du bzier.
     *  \param cx1 : Coordonne sur l'axe x du point de controle.
     *  \param cy1 : Coordonne sur l'axe y du point de controle.
     *  \param x2 : Coordonne sur l'axe x du dernier point du bzier.
     *  \param y2 : Coordonne sur l'axe y du dernier point du bzier.
     */
	void	addBezierQuadratique(double x1, double y1, double cx1, double cy1, double x2, double y2);

	/*!
     *  \brief Ajout d'llipse
     *
     *  Methode qui permet d'ajouter une llipse  la liste ellipseListe.
	 *
     *  \param x1 : Coordonne sur l'axe x du premier point de l'llipse.
     *  \param y1 : Coordonne sur l'axe y du premier point de l'llipse.
     *  \param rx : Longueur de l'axe x de l'llipse.
     *  \param ry : Longueur de l'axe y de l'llipse.
     *  \param xRotation : Angle de rotation de l'llipse sur l'axe x.
     *  \param largeArcFrag : Valeur du flag de grand arc.
     *  \param sweepFlag : Valeur du Sweep Frag.
     *  \param x2 : Coordonne sur l'axe x du dernier point de l'llipse.
     *  \param y2 : Coordonne sur l'axe y du dernier point de l'llipse.
     */
	void	addEllipse(double x1, double y1, double rx, double ry, double xRotation, bool largeArcFrag, bool sweepFlag, double x2, double y2);

	/*!
     *  \brief Ajout d'llipse
     *
     *  Methode qui permet d'ajouter un point  la liste pointListe.
	 *
     *  \param x : Coordonne sur l'axe x du point.
     *  \param y : Coordonne sur l'axe y du point.
     */
	void	addPoint(double x, double y);

	/*!
     *  \brief Ajout de Path
     *
     *  Methode qui permet d'ajouter les forme contenus dans un path. Le
	 *  path est ainsi extrait et dcompos en formes simple : lignes,
	 *  cercles, bziers cubiques, bziers quadratiques et llipses. Ces
	 *  formes sont ensuite ajoutes aux lignes correspondantes.
	 *
     *  \param strpath : Chaine contenant les informations du path  parser.
     */
	int		addPath(const char *strpath);

	/*!
     *  \brief Echantillonner les points
     *
     *  Methode qui permet d'chantillonner les points sur l'ensembles des
	 *  formes en appelant les mthodes SamplingPointsLine,
	 *  SamplingPointsCircle, SamplingPointBezierCubique,
	 *  SamplingPointBezierQuadratique et SamplingPointsEllispse.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	SamplingPoints (double distance);

	/*!
     *  \brief Echantillonner les points des lignes
     *
     *  Methode qui permet d'chantillonner les points des lignes contenues
	 *  dans la liste ligneListe. Les points son ensuite ajouts  la liste
	 *  pointListe.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	SamplingPointsLine (double distance);

	/*!
     *  \brief Echantillonner les points des cercles
     *
     *  Methode qui permet d'chantillonner les points des cercles contenus
	 *  dans la liste cercleListe. Les points son ensuite ajouts  la liste
	 *  pointListe.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	SamplingPointsCircle (double distance);

	/*!
     *  \brief Echantillonner les points des bziers cubiques
     *
     *  Methode qui permet d'chantillonner les points des bziers cubiques
	 *  contenus dans la liste bCubiqueListe. Les points son ensuite ajouts
	 *   la liste pointListe.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	SamplingPointBezierCubique (double distance);

	/*!
     *  \brief Echantillonner les points des bziers quadratiques
     *
     *  Methode qui permet d'chantillonner les points des bziers
	 *  quadratiques contenus dans la liste bQuadratiqueListe. Les points
	 *  son ensuite ajouts  la liste pointListe.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	SamplingPointBezierQuadratique (double distance);

	/*!
     *  \brief Echantillonner les points des llpses
     *
     *  Methode qui permet d'chantillonner les points des llipses
	 *  contenues dans la liste ellipseListe. Les points son ensuite ajouts
	 *   la liste pointListe.
	 *
     *  \param distance : Distance fixe entre chaque point  chantillonner.
     */
	void	SamplingPointsEllispse (double distance);

};

#endif // SAMPLINGSYMBOLE_H