/*!
 * \file TDClasse.cpp
 * \brief Fichier contenant la classe principale  instancier pour le calcul de la transforme
 *		  de distance.           
 * \author Christophe Guinchard
 * \version 1.0
 */

#include "TDClasse.h"

TDClasse::TDClasse(void)
{
	p_ImageBinaire = new TDImageBinaire;
	p_ImageMap = new TDImageMap;
}

TDClasse::~TDClasse(void)
{
	delete p_ImageBinaire;
	delete p_ImageMap;
}

int TDClasse::chargerTif(char *chemin)
{
	uint16 photometric;
	uint32 imagewidth, imagelength;

	//Ouverture du fichier
	TIFF* tif = TIFFOpen(chemin, "r");

	if (tif) {
		uint16 bitpersample; TIFFGetField(tif, TIFFTAG_BITSPERSAMPLE, &bitpersample);
		if (bitpersample == 1) {
			
			// On rcupre les donnes dans le tag TIFF
			TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &imagewidth);
			TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &imagelength);
			TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &photometric);

			// Initialisation largeur et hauteur
			largeur = (unsigned int) imagewidth;
			hauteur = (unsigned int) imagelength;

			// Initialisation des images
			p_ImageBinaire->initImageBinaire(largeur,hauteur);
			p_ImageMap->initImageMap(largeur,hauteur);

			// Initialisation des valeurs
			unsigned int p, pmax = TIFFScanlineSize(tif); 
			unsigned char* buf = (unsigned char*)_TIFFmalloc(pmax);
			unsigned int x;
			unsigned char mask; short unsigned int weight;
			
			for (unsigned int y=0; y <hauteur; y++) {			
				TIFFReadScanline(tif,buf,y);
				x=0;
				// Rcupration des donnes
				for(p=0;p<pmax-1;p++) {
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x80)>>7)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x40)>>6)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x20)>>5)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x10)>>4)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x08)>>3)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x04)>>2)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,(((buf[p] & 0x02)>>1)^photometric) ? false : true); x++;
					p_ImageBinaire->setPixel(x,y,((buf[p] & 0x01)^photometric) ? false : true); x++;
					bool test = p_ImageBinaire->getPixel(x,y);
				}

				// Rcupration du dernier caractre
				mask = 0x80; weight = 7;
				while(x!=largeur) {
					p_ImageBinaire->setPixel(x,y,(((buf[p] & mask)>>weight)^photometric) ? false : true); x++;
					mask>>= 1; weight--;
				}
			}
			_TIFFfree(buf);
		}
		TIFFClose(tif);
	}
	else return 1;

	if (sqrt((double)((largeur*largeur)+(hauteur*hauteur))) > USHRT_MAX)
		return 2;
	else
		return 0;
}

void TDClasse::squelettisation(void)
{
	int H = hauteur;
	int W = largeur;

	// On applique l'algorithme de Tohm
	for(int x = 1 ; x < W ; x++)
	{
		for(int y = 1 ; y < H ; y++)
		{
			// On vrifie les 16 cas de l'algorithme de Tohm
			for (int parcour = 1 ; parcour <= 16 && !p_ImageBinaire->getPixel(x,y) ; parcour++)
			{
				switch (parcour)
				{
				case 1 :
					if (!p_ImageBinaire->getPixel(x-1,y) && !p_ImageBinaire->getPixel(x+1,y)  && !p_ImageBinaire->getPixel(x,y-1) && p_ImageBinaire->getPixel(x,y+1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 2 :
					if (p_ImageBinaire->getPixel(x-1,y) && !p_ImageBinaire->getPixel(x+1,y)  && !p_ImageBinaire->getPixel(x,y-1) && !p_ImageBinaire->getPixel(x,y+1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 3 :
					if (!p_ImageBinaire->getPixel(x-1,y) && !p_ImageBinaire->getPixel(x+1,y)  && p_ImageBinaire->getPixel(x,y-1) && !p_ImageBinaire->getPixel(x,y+1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 4 :
					if (!p_ImageBinaire->getPixel(x-1,y) && p_ImageBinaire->getPixel(x+1,y)  && !p_ImageBinaire->getPixel(x,y-1) && !p_ImageBinaire->getPixel(x,y+1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 5 :
					if (p_ImageBinaire->getPixel(x-1,y-1) && p_ImageBinaire->getPixel(x-1,y)  && p_ImageBinaire->getPixel(x-1,y+1) && p_ImageBinaire->getPixel(x,y+1) && !p_ImageBinaire->getPixel(x+1,y+1) && !p_ImageBinaire->getPixel(x+1,y) && p_ImageBinaire->getPixel(x,y-1))   p_ImageBinaire->setPixel(x,y,true);
					break;
				case 6 :
					if (p_ImageBinaire->getPixel(x-1,y-1) && p_ImageBinaire->getPixel(x-1,y)  && p_ImageBinaire->getPixel(x-1,y+1) && p_ImageBinaire->getPixel(x,y+1) && !p_ImageBinaire->getPixel(x+1,y)   && !p_ImageBinaire->getPixel(x+1,y-1) && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 7 :
					if (!p_ImageBinaire->getPixel(x-1,y-1) && !p_ImageBinaire->getPixel(x-1,y)   && p_ImageBinaire->getPixel(x,y+1)   && p_ImageBinaire->getPixel(x+1,y+1) && p_ImageBinaire->getPixel(x+1,y) && p_ImageBinaire->getPixel(x+1,y-1) && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 8 :
					if (!p_ImageBinaire->getPixel(x-1,y)   && !p_ImageBinaire->getPixel(x-1,y+1) && p_ImageBinaire->getPixel(x,y+1)   && p_ImageBinaire->getPixel(x+1,y+1) && p_ImageBinaire->getPixel(x+1,y) && p_ImageBinaire->getPixel(x+1,y-1) && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 9 :
					if (p_ImageBinaire->getPixel(x-1,y-1) && p_ImageBinaire->getPixel(x-1,y)   && !p_ImageBinaire->getPixel(x,y+1)   && !p_ImageBinaire->getPixel(x+1,y+1) && p_ImageBinaire->getPixel(x+1,y)   && p_ImageBinaire->getPixel(x+1,y-1) && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 10 :
					if (p_ImageBinaire->getPixel(x-1,y-1) && p_ImageBinaire->getPixel(x-1,y)   && !p_ImageBinaire->getPixel(x-1,y+1) && !p_ImageBinaire->getPixel(x,y+1)  && p_ImageBinaire->getPixel(x+1,y)   && p_ImageBinaire->getPixel(x+1,y-1) && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 11 :
					if (p_ImageBinaire->getPixel(x-1,y)   && p_ImageBinaire->getPixel(x-1,y+1) && p_ImageBinaire->getPixel(x,y+1)   && p_ImageBinaire->getPixel(x+1,y+1) && p_ImageBinaire->getPixel(x+1,y)   && !p_ImageBinaire->getPixel(x+1,y-1) && !p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 12 :
					if (!p_ImageBinaire->getPixel(x-1,y-1) && p_ImageBinaire->getPixel(x-1,y)   && p_ImageBinaire->getPixel(x-1,y+1) && p_ImageBinaire->getPixel(x,y+1)   && p_ImageBinaire->getPixel(x+1,y+1) && p_ImageBinaire->getPixel(x+1,y)   && !p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 13 :
					if (p_ImageBinaire->getPixel(x-1,y)   && p_ImageBinaire->getPixel(x-1,y+1) && p_ImageBinaire->getPixel(x,y+1) && !p_ImageBinaire->getPixel(x+1,y)   && !p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 14 :
					if (p_ImageBinaire->getPixel(x-1,y-1) && p_ImageBinaire->getPixel(x-1,y) && !p_ImageBinaire->getPixel(x,y+1) && !p_ImageBinaire->getPixel(x+1,y)  && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 15 :
					if (!p_ImageBinaire->getPixel(x-1,y)   && !p_ImageBinaire->getPixel(x,y+1)   && p_ImageBinaire->getPixel(x+1,y)   && p_ImageBinaire->getPixel(x+1,y-1) && p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				case 16 :
					if (!p_ImageBinaire->getPixel(x-1,y)   && p_ImageBinaire->getPixel(x,y+1)   && p_ImageBinaire->getPixel(x+1,y+1) && p_ImageBinaire->getPixel(x+1,y)   && !p_ImageBinaire->getPixel(x,y-1)) p_ImageBinaire->setPixel(x,y,true);
					break;
				}
			}
		}
	}
}

void TDClasse::calculDT(void)
{
	unsigned int x,y,parx,pary;

	// Premier parcours : gauche  droite, haut en bas
	for (y=0 ; y < hauteur ; y++)
	{
		for (x=0 ; x < largeur ; x++)
		{
			// Si on parcours un pixel noir, la distance est nulle
			if (p_ImageBinaire->getPixel(x,y) == false)
				p_ImageMap->setPixel(x,y,0);

			//Sinon, on se base sur la valeur des voisins du haut et de gauche pour dterminernla distance minimal
			else
			{
				// On rcupre la valeur des voisins
				unsigned short int gauche;
				unsigned short int haut;
				if(x) gauche = p_ImageMap->getPixel(x-1,y);
				if(y) haut = p_ImageMap->getPixel(x,y-1);

				// Si on se trouve dans le coin de l'image
				if (x == 0 && y == 0)
					p_ImageMap->setPixel(x,y,65535);

				// Si on se trouve sur le bord gauche de l'image
				else if (x == 0)
					//p_ImageMap->setPixel(x,y,haut+1);
					p_ImageMap->setPixel(x,y,65535);

				// Si on se trouve sur le bord haut de l'image
				else if (y == 0)
					//p_ImageMap->setPixel(x,y,gauche+1);
					p_ImageMap->setPixel(x,y,65535);

				else if (gauche == 65535 && haut == 65535)
					p_ImageMap->setPixel(x,y,65535);

				// Si le pixel de gauche est plus proche d'un pixel noir, on utilisera sa valeur
				else if (gauche < haut && gauche != 65535)
					p_ImageMap->setPixel(x,y,gauche+1);

				// Sinon on utilise la valeur du pixel de du haut
				else
					p_ImageMap->setPixel(x,y,haut+1);
			}
		}
	}

	// Second parcours : droite  gauche, bas en haut
	for (pary=0 ; pary < hauteur ; pary++)
	{
		y = hauteur - pary - 1;
		for (parx=0 ; parx < largeur ; parx++)
		{
			x = largeur - parx - 1;

			// Si on ne se trouve pas sur un pixel dtect comme noir prcedement
			if (p_ImageMap->getPixel(x,y))
			{
				// On rcupre la valeur des voisins
				unsigned short int droite;
				unsigned short int bas;
				if(parx) droite = p_ImageMap->getPixel(x+1,y);
				if(pary) bas = p_ImageMap->getPixel(x,y+1);

				// Si le pixel de droite est plus proche d'un pixel blanc, on utilisera sa valeur
				if (parx && droite < p_ImageMap->getPixel(x,y))
					p_ImageMap->setPixel(x,y,droite+1);

				// Si le pixel du bas est plus proche d'un pixel blanc, on utilisera sa valeur
				if (pary && bas < p_ImageMap->getPixel(x,y))
					p_ImageMap->setPixel(x,y,bas+1);
			}
		}
	}
}

int TDClasse::transformeeDistance(char *chemin)
{
	int ret = chargerTif(chemin);
	squelettisation();
	calculDT();

	return ret;
}

int TDClasse::ecrireFichierTD(char *path)
{
	FILE* fichier = NULL;
	fichier = fopen(path,"wb");
	if (fichier == NULL) return 1;

	fwrite( p_ImageMap->getPixel() , sizeof(* p_ImageMap->getPixel()) , p_ImageMap->getHauteur()*p_ImageMap->getLargeur() , fichier);

	fclose(fichier);

	return 0;
}