#ifndef TDCLASSE_H
#define TDCLASSE_H

/*!
 * \file TDClasse.h
 * \brief Fichier contenant la classe principale  instancier pour le calcul de la transforme
 *		  de distance.           
 * \author Christophe Guinchard
 * \version 1.0
 */

#include "TDImageBinaire.h"
#include "TDImageMap.h"
#include <string.h>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#include "tiffio.h"
#include <math.h>

using namespace std;

/*! \class TDClasse
 * \brief Classe principale  instancier pour le calcul de la transforme
 *		  de distance. 
 *
 *  La classe rcupre un lien vers le fichier TIFF binaire et l'utilise
 *  pour gnrer une instance de la classe TDImageBinaire. L'image binaire
 *  est ensuite squelettis. Puis une instance de la classe TDImageMap est
 *  gnre en calculant les distances sur l'image squelttise. La carte
 *  des distances ainsi calcule peut tre crite dans un fichier SSDT
 *  avec la mthode ecrireFichierTD.
 */
class TDClasse
{

private:
	TDImageBinaire	*p_ImageBinaire;
	TDImageMap		*p_ImageMap;
	unsigned int	largeur;
	unsigned int	hauteur;

public:

    /*!
     *  \brief Constructeur
     *
     *  Constructeur de la classe TDClasse
     */
	TDClasse(void);

    /*!
     *  \brief Destructeur
     *
     *  Destructeur de la classe TDClasse
     */
	~TDClasse(void);

public:

	/*!
     *  \brief Chargement d'un fichier TIFF binaire
     *
     *  Methode qui permet d'ouvrir un fichier TIFF binaire et d'crire
	 *  les insformations contenu dans l'instance p_ImageBinaire.
     *
     *  \param chemin : Chemin vers le fichier TIFF binaire
     *  \return 0 si le fichier TIFF binaire a pu se charger sans problme.
	 *  Retourne 1 si le fichier n'a pas pu tre ouvert. Retourne 2 si le
	 *  fichier est est trop grand : la taille de la diagonale doit tre
	 *  infrieure  la valeure max d'un unsigned short.
     */
	int		chargerTif(char *chemin);

	/*!
     *  \brief Squelttisation de l'image binaire
     *
     *  Methode qui permet de squelettiser l'iage binaire contenu dans
	 *  l'instance p_ImageBinaire. La squelttisation suit la mthode de
	 *  Tohm.
     */
	void	squelettisation(void);

	/*!
     *  \brief Calcul de la transforme de distance
     *
     *  Methode qui permet de calculer la transforme de distance  partir
	 *  de l'instance p_ImageBinaire. La distance ainsi calcule pour
	 *  chaque pixel est stocke dans l'instance p_ImageMap.
     */
	void	calculDT(void);

	/*!
     *  \brief Ralisation de l'ensemble des oprations de calcul
     *
     *  Methode qui va excuter l'ensemble des mthodes ncessaire pour
	 *  calculer la transforme de distance. Elle excute ainsi les
	 *  trois mthodes : chargerTif, squelettisation et calculDT. Sa
	 *  valeur de retour est la mme que la mthode chargerTif.
	 *
	 *  \param chemin : Chemin vers le fichier TIFF binaire
     *  \return 0 si le fichier TIFF binaire a pu se charger sans problme.
	 *  Retourne 1 si le fichier n'a pas pu tre ouvert. Retourne 2 si le
	 *  fichier est est trop grand : la taille de la diagonale doit tre
	 *  infrieure  la valeure max d'un unsigned short.
     */
	int		transformeeDistance(char *chemin);

	/*!
     *  \brief Ecriture du fichier SSDT
     *
     *  Methode qui va crire chacune des distances dans un fichier SSDT
	 *
	 *  \param chemin : Chemin vers le fichier SSDT  crire.
     *  \return 0 si le fichier a correctement t crit, sinon 1.
     */
	int		ecrireFichierTD(char *path);
};

#endif // TDCLASSE_H